@@ -1,5 +1,4 @@
-"""Classes implementing generalized linear regression estimators."""
-
+"""Classes implementing generalized linear regression Regressors."""
 from ._adaptive_lasso import (
     AdaptiveGroupLasso,
     AdaptiveLasso,
