@@ -4,13 +4,11 @@ __author__ = "Fengyu Xie"
 import numbers
 import re
 import time
-import warnings
 from collections import defaultdict
 from copy import deepcopy
 from itertools import product
 
 import numpy as np
-from joblib import Parallel
 from sklearn.base import clone, is_classifier
 from sklearn.metrics import check_scoring
 from sklearn.metrics._scorer import _check_multimetric_scoring
@@ -22,7 +20,7 @@ from sklearn.model_selection._validation import (
     _insert_error_scores,
     _warn_or_raise_about_fit_failures,
 )
-from sklearn.utils.fixes import delayed
+from sklearn.utils.parallel import Parallel, delayed  # sklearn>=1.2.1
 from sklearn.utils.validation import _check_fit_params, indexable
 
 
@@ -39,6 +37,124 @@ class GridSearchCV(_GridSearchCV):
     supports "max_score" (default), which is to maximize the score;
     also supports "one_std_score", which is to apply one-standard-error
     rule to the score.
+
+    Args:
+        estimator (Estimator):
+            A object of that type is instantiated for each grid point.
+            This is assumed to implement the scikit-learn estimator
+            interface. Either estimator needs to provide a ``score``
+            function, or ``scoring`` must be passed.
+        param_grid (dict or list[dict]):
+            Dictionary representing grid of hyper-parameters with their
+            names as keys and possible values. If given as a list of
+            multiple dicts, will search on multiple grids in parallel.
+        opt_selection_method (str, default="max_score"):
+            The method to select optimal hyper params. Default to
+            "max_score", which means to maximize the score. Can also choose
+            "one_std_score", which means to apply one standard error rule
+            on scores.
+        scoring (str, callable, list, tuple or dict,
+        default="neg_root_mean_squared_error"):
+            Strategy to evaluate the performance of the cross-validated
+            model on the test set.
+            If `scoring` represents a single score, one can use:
+            - a single string (see :ref:`scoring_parameter`);
+            - a callable (see :ref:`scoring`) that returns a single value.
+            If `scoring` represents multiple scores, one can use:
+            - a list or tuple of unique strings;
+            - a callable returning a dictionary where the keys are the
+            metric names and the values are the metric scores;
+            - a dictionary with metric names as keys and callables a
+            values.
+            See :ref:`multimetric_grid_search` for an example.
+            In sparse-lm, using "neg_root_mean_squared_error" is default
+            because in cluster expansion it is more conventional to
+            analyze and present errors in the root-mean-square error format
+            compared to the r2_score.
+        n_jobs (int, default=None):
+            Number of jobs to run in parallel.
+            ``None`` means 1 unless in a :obj:`joblib.parallel_backend`
+            context.
+            ``-1`` means using all processors. See
+            :term:`Glossary <n_jobs>` for more details.
+        refit (bool, str, or callable, default=True)
+            Refit an estimator using the best found parameters on the whole
+            dataset.
+            For multiple metric evaluation, this needs to be a `str`
+            denoting the scorer that would be used to find the best
+            parameters for refitting the estimator at the end.
+            Where there are considerations other than maximum score in
+            choosing a best estimator, ``refit`` can be set to a function
+            which returns the selected ``best_index_`` given
+            ``cv_results_``.
+            In that case, the ``best_estimator_`` and ``best_params_`` will
+            be set according to the returned ``best_index_`` while the
+            ``best_score_`` attribute will not be available.
+            The refitted estimator is made available at the
+            ``best_estimator_`` attribute and permits using ``predict``
+            directly on this instance.
+            Also for multiple metric evaluation, the attributes
+            ``best_index_``, ``best_score_`` and ``best_params_`` will only
+            be available if ``refit`` is set and all of them will be
+            determined w.r.t this specific scorer.
+            See ``scoring`` parameter to know more about multiple metric
+            evaluation.
+        cv (int, cross-validation generator or an iterable, default=None):
+            Determines the cross-validation splitting strategy.
+            Possible inputs for cv are:
+            - None, to use the default 5-fold cross validation,
+            - integer, to specify the number of folds in a
+            `(Stratified)KFold`,
+            - :term:`CV splitter`,
+            - An iterable yielding (train, test) splits as arrays of
+            indices.
+            For integer/None inputs, if the estimator is a classifier and
+            ``y`` is either binary or multiclass, :class:`StratifiedKFold`
+            is used. In all other cases, :class:`KFold` is used. These
+            splitters are instantiated with `shuffle=False` so the splits
+            will be the same across calls.
+            Refer :ref:`User Guide <cross_validation>` for the various
+            cross-validation strategies that can be used here.
+            Notice that if cv is not specified, KFold(5) will be used, and your
+            training set will not be shuffled in each train-test split. This can
+            be dangerous if your training set has internal relations, for example,
+            structures with similar composition are close to each other in feature
+            rows. In this case, you should use RepeatedKFold as cv instead.
+        verbose (int, default=0):
+            Controls the verbosity: the higher, the more messages.
+            - >1 : the computation time for each fold and parameter
+            candidate is displayed;
+            - >2 : the score is also displayed;
+            - >3 : the fold and candidate parameter indexes are also
+            displayed together with the starting time of the computation.
+        pre_dispatch (int, or str, default='2*n_jobs'):
+            Controls the number of jobs that get dispatched during parallel
+            execution. Reducing this number can be useful to avoid an
+            explosion of memory consumption when more jobs get dispatched
+            than CPUs can process. This parameter can be:
+                - None, in which case all the jobs are immediately
+                  created and spawned. Use this for lightweight and
+                  fast-running jobs, to avoid delays due to on-demand
+                  spawning of the jobs
+                - An int, giving the exact number of total jobs that are
+                  spawned
+                - A str, giving an expression as a function of n_jobs,
+                  as in '2*n_jobs'
+        error_score ('raise' or numeric, default=np.nan):
+            Value to assign to the score if an error occurs in estimator
+            fitting. If set to 'raise', the error is raised. If a numeric
+            value is given, FitFailedWarning is raised. This parameter
+            does not affect the refit step, which will always raise the
+            error.
+        return_train_score (bool, default=False):
+            If ``False``, the ``cv_results_`` attribute will not include
+            training scores.
+            Computing training scores is used to get insights on how
+            different parameter settings impact the
+            overfitting/underfitting trade-off. However, computing the
+            scores on the training set can be computationally expensive and
+            is not strictly required to select the parameters that yield
+            the best generalization performance.
     """
 
     def __init__(
@@ -56,110 +172,6 @@ class GridSearchCV(_GridSearchCV):
         error_score=np.nan,
         return_train_score=False,
     ):
-        """Initialize CVSearch tool.
-
-        Args:
-            estimator (Estimator):
-                A object of that type is instantiated for each grid point.
-                This is assumed to implement the scikit-learn estimator interface.
-                Either estimator needs to provide a ``score`` function,
-                or ``scoring`` must be passed.
-            param_grid (dict or list[dict]):
-                Dictionary representing grid of hyper-parameters with their names
-                as keys and possible values. If given as a list of multiple dicts,
-                will search on multiple grids in parallel.
-            opt_selection_method (str, default="max_score"):
-                The method to select optimal hyper params. Default to "max_score", which
-                means to maximize the score. Can also choose "one_std_score", which means
-                to apply one standard error rule on scores.
-            scoring (str, callable, list, tuple or dict, default="neg_root_mean_squared_error"):
-                Strategy to evaluate the performance of the cross-validated model on
-                the test set.
-                If `scoring` represents a single score, one can use:
-                - a single string (see :ref:`scoring_parameter`);
-                - a callable (see :ref:`scoring`) that returns a single value.
-                If `scoring` represents multiple scores, one can use:
-                - a list or tuple of unique strings;
-                - a callable returning a dictionary where the keys are the metric
-                  names and the values are the metric scores;
-                - a dictionary with metric names as keys and callables a values.
-                See :ref:`multimetric_grid_search` for an example.
-                Note: In sparse-lm, using "neg_root_mean_squared_error" is default
-                because in cluster expansion it is more conventional to analyze and present
-                errors in the root-mean-square error format compared to the r2_score.
-            n_jobs (int, default=None):
-                Number of jobs to run in parallel.
-                ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
-                ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
-                for more details.
-            refit (bool, str, or callable, default=True)
-                Refit an estimator using the best found parameters on the whole
-                dataset.
-                For multiple metric evaluation, this needs to be a `str` denoting the
-                scorer that would be used to find the best parameters for refitting
-                the estimator at the end.
-                Where there are considerations other than maximum score in
-                choosing a best estimator, ``refit`` can be set to a function which
-                returns the selected ``best_index_`` given ``cv_results_``. In that
-                case, the ``best_estimator_`` and ``best_params_`` will be set
-                according to the returned ``best_index_`` while the ``best_score_``
-                attribute will not be available.
-                The refitted estimator is made available at the ``best_estimator_``
-                attribute and permits using ``predict`` directly on this
-                instance.
-                Also for multiple metric evaluation, the attributes ``best_index_``,
-                ``best_score_`` and ``best_params_`` will only be available if
-                ``refit`` is set and all of them will be determined w.r.t this specific
-                scorer.
-                See ``scoring`` parameter to know more about multiple metric
-                evaluation.
-            cv (int, cross-validation generator or an iterable, default=None):
-                Determines the cross-validation splitting strategy.
-                Possible inputs for cv are:
-                - None, to use the default 5-fold cross validation,
-                - integer, to specify the number of folds in a `(Stratified)KFold`,
-                - :term:`CV splitter`,
-                - An iterable yielding (train, test) splits as arrays of indices.
-                For integer/None inputs, if the estimator is a classifier and ``y`` is
-                either binary or multiclass, :class:`StratifiedKFold` is used. In all
-                other cases, :class:`KFold` is used. These splitters are instantiated
-                with `shuffle=False` so the splits will be the same across calls.
-                Refer :ref:`User Guide <cross_validation>` for the various
-                cross-validation strategies that can be used here.
-            verbose (int, default=0):
-                Controls the verbosity: the higher, the more messages.
-                - >1 : the computation time for each fold and parameter candidate is
-                  displayed;
-                - >2 : the score is also displayed;
-                - >3 : the fold and candidate parameter indexes are also displayed
-                  together with the starting time of the computation.
-            pre_dispatch (int, or str, default='2*n_jobs'):
-                Controls the number of jobs that get dispatched during parallel
-                execution. Reducing this number can be useful to avoid an
-                explosion of memory consumption when more jobs get dispatched
-                than CPUs can process. This parameter can be:
-                    - None, in which case all the jobs are immediately
-                      created and spawned. Use this for lightweight and
-                      fast-running jobs, to avoid delays due to on-demand
-                      spawning of the jobs
-                    - An int, giving the exact number of total jobs that are
-                      spawned
-                    - A str, giving an expression as a function of n_jobs,
-                      as in '2*n_jobs'
-            error_score ('raise' or numeric, default=np.nan):
-                Value to assign to the score if an error occurs in estimator fitting.
-                If set to 'raise', the error is raised. If a numeric value is given,
-                FitFailedWarning is raised. This parameter does not affect the refit
-                step, which will always raise the error.
-            return_train_score (bool, default=False):
-                If ``False``, the ``cv_results_`` attribute will not include training
-                scores.
-                Computing training scores is used to get insights on how different
-                parameter settings impact the overfitting/underfitting trade-off.
-                However, computing the scores on the training set can be computationally
-                expensive and is not strictly required to select the parameters that
-                yield the best generalization performance.
-        """
         super().__init__(
             estimator=estimator,
             param_grid=param_grid,
@@ -195,8 +207,8 @@ class GridSearchCV(_GridSearchCV):
             metrics = results[f"mean_test_{refit_metric}"]
             param_names = [key for key in results if re.match(r"^param_(\w+)", key)]
             params = []
-            # Will only apply one std rule on numerical, all positive parameters,
-            # which are usually regularization factors.
+            # Will only apply one std rule on numerical, all positive
+            # parameters, which are usually regularization factors.
             # All parameters are equally treated in grid and line search.
             for name in param_names:
                 if all(isinstance(val, numbers.Number) for val in results[name]):
@@ -219,22 +231,25 @@ class GridSearchCV(_GridSearchCV):
             X(array-like of shape (n_samples, n_features)):
                 Training vector, where `n_samples` is the number of samples and
                 `n_features` is the number of features.
-            y(array-like of shape (n_samples, n_output) or (n_samples,), default=None):
+            y(array-like of shape (n_samples, n_output) or (n_samples,),
+            default=None):
                 Target relative to X for classification or regression;
                 None for unsupervised learning.
             groups(array-like of shape (n_samples,), default=None):
-                Group labels for the samples used while splitting the dataset into
-                train/test set. Only used in conjunction with a "Group" :term:`cv`
-                instance (e.g., :class:`~sklearn.model_selection.GroupKFold`).
+                Group labels for the samples used while splitting the dataset
+                into train/test set. Only used in conjunction with a "Group"
+                :term:`cv` instance
+                (e.g., :class:`~sklearn.model_selection.GroupKFold`).
             **fit_params:
                 Parameters passed to the `fit` method of the estimator.
                 If a fit parameter is an array-like whose length is equal to
-                `num_samples` then it will be split across CV groups along with `X`
-                and `y`. For example, the :term:`sample_weight` parameter is split
-                because `len(sample_weights) = len(X)`.
-            Returns:
-                self(GridSearch):
-                    Instance of fitted estimator.
+                `num_samples` then it will be split across CV groups along with
+                `X` and `y`. For example, the :term:`sample_weight` parameter
+                is split because `len(sample_weights) = len(X)`.
+
+        Returns:
+            self(GridSearch):
+                Instance of fitted estimator.
         """
         estimator = self.estimator
         refit_metric = "score"
@@ -299,8 +314,12 @@ class GridSearchCV(_GridSearchCV):
                         candidate_progress=(cand_idx, n_candidates),
                         **fit_and_score_kwargs,
                     )
-                    for (cand_idx, parameters), (split_idx, (train, test)) in product(
-                        enumerate(candidate_params), enumerate(cv.split(X, y, groups))
+                    for (cand_idx, parameters), (
+                        split_idx,
+                        (train, test),
+                    ) in product(
+                        enumerate(candidate_params),
+                        enumerate(cv.split(X, y, groups)),
                     )
                 )
 
@@ -435,90 +454,106 @@ class LineSearchCV(BaseSearchCV):
         Args:
             estimator (Estimator):
                 A object of that type is instantiated for each grid point.
-                This is assumed to implement the scikit-learn estimator interface.
-                Either estimator needs to provide a ``score`` function,
-                or ``scoring`` must be passed.
+                This is assumed to implement the scikit-learn estimator
+                interface. Either estimator needs to provide a ``score``
+                function, or ``scoring`` must be passed.
             param_grid (list[tuple]):
                 List of tuples with parameters names (`str`) as first element
                 and lists of parameter settings to try as the second element.
-                In LineSearch, the hyper-params given first will be searched first
-                in a cycle. Multiple grids search is NOT supported!
+                In LineSearch, the hyper-params given first will be searched
+                first in a cycle. Multiple grids search is NOT supported!
             opt_selection_method (list(str) or str, default="max_score"):
-                The method to select optimal hyper params. Default to "max_score", which
-                means to maximize score score. Can also choose "one_std_score", which means
-                to apply one standard error rule on score scores.
-                In line search, this argument can also be given as a list of str. This
-                will allow different selection methods for corresponding hyper-params in
-                the param_grid. For example, a good practice when using L2L0 estimator
-                shall be opt_selection_method = ["one_std_score", "max_score"] for "alpha"
-                and "l0_ratio", respectively.
+                The method to select optimal hyper params. Default to
+                "max_score", which means to maximize score score. Can also
+                choose "one_std_score", which means to apply one standard
+                error rule on score scores.
+                In line search, this argument can also be given as a list
+                of str. This will allow different selection methods for the
+                corresponding hyper-params in ``param_grid``. For example,
+                a good practice when using L2L0 estimator could be
+                ``opt_selection_method`` = ["one_std_score", "max_score"]
+                for "alpha" and "l0_ratio", respectively.
             n_iter (int, default=None):
-                Number of iterations to perform. One iteration means a 1D search on
-                one hyper-param, and we scan one hyper-param at a time in the order of
-                param_grid.
-                n_iter must be at least as large as the number of hyper-params. Default
-                is 2 * number of hyper-params.
-            scoring (str, callable, list, tuple or dict, default="neg_root_mean_squared_error"):
-                Strategy to evaluate the performance of the cross-validated model on
-                the test set.
+                Number of iterations to perform. One iteration means a 1D
+                search on one hyper-param, and we scan one hyper-param at
+                a time in the order of param_grid.
+                n_iter must be at least as large as the number of hyper-params.
+                Default is 2 * number of hyper-params.
+            scoring (str, callable, list, tuple or dict):
+                Strategy to evaluate the performance of the cross-validated model on the test set.
+            default="neg_root_mean_squared_error"):
+                Strategy to evaluate the performance of the cross-validated
+                model on the test set.
                 If `scoring` represents a single score, one can use:
                 - a single string (see :ref:`scoring_parameter`);
                 - a callable (see :ref:`scoring`) that returns a single value.
                 If `scoring` represents multiple scores, one can use:
                 - a list or tuple of unique strings;
-                - a callable returning a dictionary where the keys are the metric
-                  names and the values are the metric scores;
-                - a dictionary with metric names as keys and callables a values.
+                - a callable returning a dictionary where the keys are the
+                metric names and the values are the metric scores;
+                - a dictionary with metric names as keys and callables a
+                values.
                 See :ref:`multimetric_grid_search` for an example.
-                Note: In sparse-lm, using "neg_root_mean_squared_error" is default
-                because in cluster expansion it is more conventional to analyze and present
-                errors in the root-mean-square error format compared to the r2_score.
+                In sparse-lm, using "neg_root_mean_squared_error" is default
+                because in cluster expansion it is more conventional to
+                analyze and present errors in the root-mean-square error format
+                compared to the r2_score.
             n_jobs (int, default=None):
                 Number of jobs to run in parallel.
-                ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
-                ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
-                for more details.
-            refit (bool, str, or callable, default=True)
+                ``None`` means 1 unless in a :obj:`joblib.parallel_backend`
+                context.
+                ``-1`` means using all processors. See
+                :term:`Glossary <n_jobs>` for more details.
+            refit (bool, str, or callable, default=True):
                 Refit an estimator using the best found parameters on the whole
                 dataset.
-                For multiple metric evaluation, this needs to be a `str` denoting the
-                scorer that would be used to find the best parameters for refitting
-                the estimator at the end.
+                For multiple metric evaluation, this needs to be a `str`
+                denoting the scorer that would be used to find the best
+                parameters for refitting the estimator at the end.
                 Where there are considerations other than maximum score in
-                choosing a best estimator, ``refit`` can be set to a function which
-                returns the selected ``best_index_`` given ``cv_results_``. In that
-                case, the ``best_estimator_`` and ``best_params_`` will be set
-                according to the returned ``best_index_`` while the ``best_score_``
-                attribute will not be available.
-                The refitted estimator is made available at the ``best_estimator_``
-                attribute and permits using ``predict`` directly on this
-                instance.
-                Also for multiple metric evaluation, the attributes ``best_index_``,
-                ``best_score_`` and ``best_params_`` will only be available if
-                ``refit`` is set and all of them will be determined w.r.t this specific
-                scorer.
+                choosing a best estimator, ``refit`` can be set to a function
+                which returns the selected ``best_index_`` given
+                ``cv_results_``.
+                In that case, the ``best_estimator_`` and ``best_params_`` will
+                be set according to the returned ``best_index_`` while the
+                ``best_score_`` attribute will not be available.
+                The refitted estimator is made available at the
+                ``best_estimator_`` attribute and permits using ``predict``
+                directly on this instance.
+                Also for multiple metric evaluation, the attributes
+                ``best_index_``, ``best_score_`` and ``best_params_`` will only
+                be available if ``refit`` is set and all of them will be
+                determined w.r.t this specific scorer.
                 See ``scoring`` parameter to know more about multiple metric
                 evaluation.
             cv (int, cross-validation generator or an iterable, default=None):
                 Determines the cross-validation splitting strategy.
                 Possible inputs for cv are:
                 - None, to use the default 5-fold cross validation,
-                - integer, to specify the number of folds in a `(Stratified)KFold`,
+                - integer, to specify the number of folds in a
+                `(Stratified)KFold`,
                 - :term:`CV splitter`,
-                - An iterable yielding (train, test) splits as arrays of indices.
-                For integer/None inputs, if the estimator is a classifier and ``y`` is
-                either binary or multiclass, :class:`StratifiedKFold` is used. In all
-                other cases, :class:`KFold` is used. These splitters are instantiated
-                with `shuffle=False` so the splits will be the same across calls.
+                - An iterable yielding (train, test) splits as arrays of
+                indices.
+                For integer/None inputs, if the estimator is a classifier and
+                ``y`` is either binary or multiclass, :class:`StratifiedKFold`
+                is used. In all other cases, :class:`KFold` is used. These
+                splitters are instantiated with `shuffle=False` so the splits
+                will be the same across calls.
                 Refer :ref:`User Guide <cross_validation>` for the various
                 cross-validation strategies that can be used here.
+                Notice that if cv is not specified, KFold(5) will be used, and your
+                training set will not be shuffled in each train-test split. This can
+                be dangerous if your training set has internal relations, for example,
+                structures with similar composition are close to each other in feature
+                rows. In this case, you should use RepeatedKFold as cv instead.
             verbose (int, default=0):
                 Controls the verbosity: the higher, the more messages.
-                - >1 : the computation time for each fold and parameter candidate is
-                  displayed;
+                - >1 : the computation time for each fold and parameter
+                candidate is displayed;
                 - >2 : the score is also displayed;
-                - >3 : the fold and candidate parameter indexes are also displayed
-                  together with the starting time of the computation.
+                - >3 : the fold and candidate parameter indexes are also
+                displayed together with the starting time of the computation.
             pre_dispatch (int, or str, default='2*n_jobs'):
                 Controls the number of jobs that get dispatched during parallel
                 execution. Reducing this number can be useful to avoid an
@@ -533,64 +568,38 @@ class LineSearchCV(BaseSearchCV):
                     - A str, giving an expression as a function of n_jobs,
                       as in '2*n_jobs'
             error_score ('raise' or numeric, default=np.nan):
-                Value to assign to the score if an error occurs in estimator fitting.
-                If set to 'raise', the error is raised. If a numeric value is given,
-                FitFailedWarning is raised. This parameter does not affect the refit
-                step, which will always raise the error.
+                Value to assign to the score if an error occurs in estimator
+                fitting. If set to 'raise', the error is raised. If a numeric
+                value is given, FitFailedWarning is raised. This parameter
+                does not affect the refit step, which will always raise the
+                error.
             return_train_score (bool, default=False):
-                If ``False``, the ``cv_results_`` attribute will not include training
-                scores.
-                Computing training scores is used to get insights on how different
-                parameter settings impact the overfitting/underfitting trade-off.
-                However, computing the scores on the training set can be computationally
-                expensive and is not strictly required to select the parameters that
-                yield the best generalization performance.
+                If ``False``, the ``cv_results_`` attribute will not include
+                training scores.
+                Computing training scores is used to get insights on how
+                different parameter settings impact the
+                overfitting/underfitting trade-off. However, computing the
+                scores on the training set can be computationally expensive and
+                is not strictly required to select the parameters that yield
+                the best generalization performance.
+
         """
         # These are equally passed into GridSearchCV objects at each iteration.
-        self.estimator = estimator
-        self.scoring = scoring
-        self.n_jobs = n_jobs
-        self.refit = refit
-        self.cv = cv
-        self.verbose = verbose
-        self.pre_dispatch = pre_dispatch
-        self.error_score = error_score
-        self.return_train_score = return_train_score
-
-        self.param_grid = param_grid
-        if isinstance(param_grid[0][0], str) and isinstance(
-            param_grid[0], (tuple, list)
-        ):
-            self.n_params = len(param_grid)
-        else:
-            raise ValueError("Parameters grid not given in the correct format!")
-
-        if opt_selection_method is None:
-            self.opt_selection_methods = ["max_score" for _ in range(self.n_params)]
-        elif isinstance(opt_selection_method, str):
-            self.opt_selection_methods = [
-                opt_selection_method for _ in range(self.n_params)
-            ]
-        elif (
-            isinstance(opt_selection_method, (list, tuple))
-            and all(isinstance(m, str) for m in opt_selection_method)
-            and len(opt_selection_method) == self.n_params
-        ):
-            self.opt_selection_methods = opt_selection_method
-        else:
-            raise ValueError(
-                "Optimal hyperparams selection method"
-                " should either not be given, or given as a single string,"
-                " or as a list or strings with the same length as parameters!"
-            )
-
-        # Set a proper value for this, not too large or too small.
-        self.n_iter = (
-            n_iter if (n_iter is not None and n_iter > 0) else 2 * self.n_params
+        super().__init__(
+            estimator=estimator,
+            scoring=scoring,
+            n_jobs=n_jobs,
+            refit=refit,
+            cv=cv,
+            verbose=verbose,
+            pre_dispatch=pre_dispatch,
+            error_score=error_score,
+            return_train_score=return_train_score,
         )
-
-        # Stores GridSearch object at each iteration
-        self._history = []
+        self.param_grid = param_grid
+        self.opt_selection_method = opt_selection_method
+        self.n_iter = n_iter
+        # Do not validate parameters in init.
 
     def fit(self, X, y=None, *, groups=None, **fit_params):
         """Run fit with all sets of parameters.
@@ -599,30 +608,65 @@ class LineSearchCV(BaseSearchCV):
             X(array-like of shape (n_samples, n_features)):
                 Training vector, where `n_samples` is the number of samples and
                 `n_features` is the number of features.
-            y(array-like of shape (n_samples, n_output) or (n_samples,), default=None):
+            y(array-like of shape (n_samples, n_output) or (n_samples,),
+            default=None):
                 Target relative to X for classification or regression;
                 None for unsupervised learning.
             groups(array-like of shape (n_samples,), default=None):
-                Group labels for the samples used while splitting the dataset into
-                train/test set. Only used in conjunction with a "Group" :term:`cv`
-                instance (e.g., :class:`~sklearn.model_selection.GroupKFold`).
+                Group labels for the samples used while splitting the dataset
+                into train/test set. Only used in conjunction with a "Group"
+                :term:`cv` instance
+                (e.g., :class:`~sklearn.model_selection.GroupKFold`).
             **fit_params:
                 Parameters passed to the `fit` method of the estimator.
                 If a fit parameter is an array-like whose length is equal to
-                `num_samples` then it will be split across CV groups along with `X`
-                and `y`. For example, the :term:`sample_weight` parameter is split
-                because `len(sample_weights) = len(X)`.
-            Returns:
-                self(LineSearch):
-                    Instance of fitted estimator.
+                `num_samples` then it will be split across CV groups along with
+                `X` and `y`. For example, the :term:`sample_weight` parameter
+                is split because `len(sample_weights) = len(X)`.
+
+        Returns:
+            self (LineSearch):
+                Instance of fitted estimator.
         """
-        if len(self._history) > 0:
-            warnings.warn("Overwrite existing fit history!")
-            self._history = []
+        # Validate parameters.
+        if isinstance(self.param_grid[0][0], str) and isinstance(
+            self.param_grid[0], (tuple, list)
+        ):
+            n_params_ = len(self.param_grid)
+        else:
+            raise ValueError("Parameter grid is not given in the correct format!")
+
+        if self.opt_selection_method is None:
+            opt_selection_methods = ["max_score" for _ in range(n_params_)]
+        elif isinstance(self.opt_selection_method, str):
+            opt_selection_methods = [
+                self.opt_selection_method for _ in range(n_params_)
+            ]
+        elif (
+            isinstance(self.opt_selection_method, (list, tuple))
+            and all(isinstance(m, str) for m in self.opt_selection_method)
+            and len(self.opt_selection_method) == n_params_
+        ):
+            opt_selection_methods = self.opt_selection_method
+        else:
+            raise ValueError(
+                "Optimal hyperparams selection methods should be given as a"
+                " single string, or as a list of strings with the same"
+                " amount of parameters!"
+            )
+
+        # Need a proper value for n_iter, not too large or too small.
+        if self.n_iter is not None and self.n_iter > 0:
+            n_iter = self.n_iter
+        else:
+            n_iter = 2 * n_params_
+
+        # Stores GridSearch object at each iteration
+        history_ = []
 
         best_line_params_ = None
-        for i in range(self.n_iter):
-            param_id = i % self.n_params
+        for i in range(n_iter):
+            param_id = i % n_params_
             if best_line_params_ is None:
                 last_params = [values[0] for name, values in self.param_grid]
             else:
@@ -639,7 +683,7 @@ class LineSearchCV(BaseSearchCV):
             grid_search = GridSearchCV(
                 estimator=self.estimator,
                 param_grid=param_line,
-                opt_selection_method=self.opt_selection_methods[param_id],
+                opt_selection_method=opt_selection_methods[param_id],
                 scoring=self.scoring,
                 n_jobs=self.n_jobs,
                 refit=self.refit,
@@ -651,16 +695,14 @@ class LineSearchCV(BaseSearchCV):
             )
             grid_search.fit(X=X, y=y, groups=groups, **fit_params)
             best_line_params_ = deepcopy(grid_search.best_params_)
-            self._history.append(grid_search)
+            history_.append(grid_search)
 
         # Buffer fitted attributes into LineSearch object.
         attrs = [
-            v
-            for v in vars(self._history[-1])
-            if v.endswith("_") and not v.startswith("__")
+            v for v in vars(history_[-1]) if v.endswith("_") and not v.startswith("__")
         ]
         for attr in attrs:
-            setattr(self, attr, getattr(self._history[-1], attr))
+            setattr(self, attr, getattr(history_[-1], attr))
         return self
 
     def _run_search(self, evaluate_candidates):
